﻿using Infragistics.Controls.Grids;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace XamGridCustomControl
{
    public class CustomXamGrid : XamGrid
    {

        public CustomXamGrid()
        {
            this.Columns.CollectionChanged += Columns_CollectionChanged;
        }

        private void Columns_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (this.ColumnsCollection != null)
            {
                if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Add)
                {
                    foreach(Column c in e.NewItems)
                    {
                        if (!this.ColumnsCollection.Contains(c))
                        {
                            this.ColumnsCollection.Add(c);
                        }
                    }
                }
                else if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Remove)
                {
                    foreach (Column c in e.OldItems)
                    {
                        if (this.ColumnsCollection.Contains(c))
                        {
                            this.ColumnsCollection.Remove(c);
                        }
                    }                            
                }
            }
        }

        public ObservableCollection<Column> ColumnsCollection
        {
            get { return (ObservableCollection<Column>)GetValue(ColumnsCollectionProperty); }
            set { SetValue(ColumnsCollectionProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ColumnsCollection.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ColumnsCollectionProperty =
            DependencyProperty.Register("ColumnsCollection", typeof(ObservableCollection<Column>), typeof(CustomXamGrid), new PropertyMetadata(null, new PropertyChangedCallback(ColumnsCollectionPropertyChanged)));

        static void ColumnsCollectionPropertyChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {
            CustomXamGrid grid = obj as CustomXamGrid;
            grid.HandlePropChange(obj, e);            
        }       

        internal void HandlePropChange(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {
            if (e.OldValue != null)
            {
                ObservableCollection<Column> oldVal = e.OldValue as ObservableCollection<Column>;
                oldVal.CollectionChanged -= NewVal_CollectionChanged;
            }

            if (e.NewValue != null)
            {
                ObservableCollection<Column> newVal = e.NewValue as ObservableCollection<Column>;
                newVal.CollectionChanged += NewVal_CollectionChanged;
                this.Columns.Clear();                

                foreach(Column c in newVal)
                {
                    this.Columns.Add(c);
                }
            }
        }

        private void NewVal_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if(e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Add)
            {
                foreach(Column c in this.ColumnsCollection)
                {
                    if (!this.Columns.Contains(c))
                    {
                        this.Columns.Add(c);
                    }
                }
            }
            else if(e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Remove)
            {
                foreach(Column c in this.Columns)
                {
                    if (!this.ColumnsCollection.Contains(c))
                    {
                        this.Columns.Remove(c);
                    }
                }
            }
        }
    }
}
